#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>

#include "models.h"
#include "util.h"

void print_help(char **argv);
void setup(klist *ctx, int argc, char **argv);

int main(int argc, char **argv) {
    klist *ctx = klist_init("test.db");

    if (argc == 1) {
        print_help(argv);
        ctx->error = 1;
    }

    if (!ctx->error) {
        setup(ctx, argc, argv);
        klist_user *user = klist_user_get_by_local(ctx, getuid());
        int i = 0;
        switch (ctx->cmd) {
            case USER:
                switch (((klist_user_context *)ctx->cmd_ctx)->cmd) {
                case USER_GET:
                    if (user) {
                        fprintf(
                            stderr,
                            "User: %p\nID: %lu\n", user->name, user->id
                        );
                        size_t lists_len, i = 0;
                        klist_list **lists = klist_list_get_all_by_user(ctx, user->id, &lists_len);
                        fprintf(stderr, "Lists: %lu\n", lists_len);
                        for (; i < lists_len; i++) klist_list_deinit(lists[i]);
                    } else fprintf(stderr, "No user for '%s' found.\n", getlogin());
                    break;
                case USER_CREATE:
                    fprintf(
                        stderr,
                        (klist_assure_user(ctx, getuid(), getlogin()))
                        ? "User created.\n"
                        : "User already exists.\n"
                     );
                    break;
                case USER_DELETE:
                    if (user) {
                        klist_user_delete(ctx, user);
                        fprintf(stderr, "User deleted.\n");
                    } else fprintf(stderr, "User not found, no changes done.\n");
                    break;
                }
            case LIST:
                klist_list_context *list_ctx = ctx->cmd_ctx;
                klist_list *list = NULL;
                if (!list_ctx->name) { fprintf(stderr, "Missing name.\n"); break; }
                switch (list_ctx->cmd) {
                case LIST_ADD:
                    if (!list_ctx->stages_len) { fprintf(stderr, "Missing stages.\n"); break; }
                    list = klist_list_init();
                    list->name = (unsigned char*)list_ctx->name;
                    list->desc = (unsigned char*)list_ctx->desc;
                    klist_list_save(ctx, list);
                    i = 0;
                    for (; i < list_ctx->stages_len; i++) {
                        klist_stage *stage = klist_stage_init();
                        stage->name = (unsigned char*)list_ctx->stages[i];
                        stage->list_id = list->id;
                        klist_stage_save(ctx, stage);
                        klist_stage_deinit(stage);
                    }
                    break;
                case LIST_EDIT:
                    fprintf(stderr, "Not implemented\n");
                    break;
                case LIST_GET:
                    klist_assure_user(ctx, getuid(), getlogin());
                    user = klist_user_get_by_local(ctx, getuid());
                    list = klist_list_get_by_user_and_name(ctx, user->id, list_ctx->name);
                    if (list) {
                        u_int tasks_len = 0;
                        klist_task **tasks = klist_task_get_for_list(ctx, list->id, &tasks_len);
                        fprintf(stderr, "Name: %s\nDescription: %s\nTasks: %d\n", list->name, list->desc, tasks_len);
                        int i = 0;
                        for(; i < tasks_len; i++) klist_task_deinit(tasks[i]);
                        free(tasks);
                    } else
                        fprintf(stderr, "List '%s' not found.\n", list_ctx->name);
                    break;
                case LIST_DELETE:
                    list = klist_list_get_by_user_and_name(ctx, user->id, list_ctx->name);
                    if (list) {
                        u_int tasks_len = 0;
                        klist_task **tasks = klist_task_get_for_list(ctx, list->id, &tasks_len);
                        int i = 0;
                        for (; i < tasks_len; i++) {
                            klist_task_delete(ctx, tasks[i]);
                            klist_task_deinit(tasks[i]);
                        }
                        free(tasks);
                    }
                }
                if (!list) klist_list_deinit(list);
            case TASK:
                switch (((klist_task_context *)ctx->cmd_ctx)->cmd) {
                case TASK_ADD:
                case TASK_EDIT:
                case TASK_GET:
                case TASK_DELETE:
                    fprintf(stderr, "Not implemented\n");
        ;
                }
            default: ;
        }
        if (user) klist_user_deinit(user);
    }

    const int error = ctx->error;
    klist_deinit(ctx);
    return error;
}

void print_help(char **argv) {
    fprintf(stderr, "Usage: %s <action>\n", argv[0]);
}

void setup(klist *ctx, int argc, char **argv) {
    if (strcmp(argv[1], "user") == 0)
        ctx->cmd = USER;
    else if (strcmp(argv[1], "list") == 0)
        ctx->cmd = LIST;
    else if (strcmp(argv[1], "task") == 0)
        ctx->cmd = TASK;
    else print_help(argv);

    int opt;
    switch (ctx->cmd) {
        case USER:
            optind = 2;
            klist_user_context *user_ctx = klist_user_context_init(ctx);
            while ((opt = getopt(argc, argv, "cdg")) != -1)
                switch (opt) {
                    case 'c':
                        user_ctx->cmd = USER_CREATE;
                        break;
                    case 'd':
                        user_ctx->cmd = USER_DELETE;
                        break;
                    case 'g':
                        user_ctx->cmd = USER_GET;
                        break;
                    default:
                        print_help(argv);
                }
            ctx->cmd_ctx = user_ctx;
            break;
        case LIST:
            if (argc < 3) {
                fprintf(stderr, "Prints all lists.\n");
                break;
            }
            klist_list_context *list_ctx = malloc(sizeof(struct klist_list_context));

            optind = 3;
            while ((opt = getopt(argc, argv, "aeds:")) != -1)
                switch (opt) {
                    case 'a':
                        list_ctx->cmd = LIST_ADD;
                        break;
                    case 'e':
                        list_ctx->cmd = LIST_EDIT;
                        break;
                    case 'd':
                        list_ctx->cmd = LIST_DELETE;
                        break;
                    default:
                        print_help(argv);
                        fprintf(stderr, ""
                                "list options:"
                                "-a\tadd a list"
                                "-e\tedit a list"
                                "-d\tdelete a list"
                                "Not providing a parameter prints all lists.");
            }
            ctx->cmd_ctx = list_ctx;
            break;
        case TASK:
            if (argc < 3) {
                fprintf(stderr, "Prints all lists.\n");
                break;
            }
            klist_list_context *task_ctx = malloc(sizeof(struct klist_task_context));

            optind = 3;

            while ((opt = getopt(argc, argv, "aeds:")) != -1)
                switch (opt) {
                    case 'a':
                        task_ctx->cmd = TASK_ADD;
                        break;
                    case 'e':
                        task_ctx->cmd = TASK_EDIT;
                        break;
                    case 'd':
                        task_ctx->cmd = TASK_DELETE;
                        break;
                    default:
                        print_help(argv);
                        fprintf(stderr, ""
                                "task options:"
                                "-a\tadd a task"
                                "-e\tedit a task"
                                "-d\tdelete a task"
                                "Not providing a parameter prints all tasks of a list.");
                }

            ctx->cmd_ctx = task_ctx;
            break;
        default:
            fprintf(stderr, "How did we land here?! Pls report argv[1] = %s\n", argv[1]);
            print_help(argv);
    }
}
